#include <libnd.h>
#include <libnd_tcpstate.h>

const char *
name(void)
{
  return "TCP-State-Tracker-Test";
}

gboolean
run(LND_Trace *unused, LND_PluginArgs *args)
{
  int i;
  LND_Trace *trace;
  LND_PacketIterator pit;
  LND_Packet *packet;
  LND_TCPConn *conn;
  GHashTable *ht;
  char line[MAXPATHLEN];
  gboolean new_conn;

  if (!args)
    return FALSE;

  if (args->argc == 0)
    {
      printf("Please provide one or more traces to scan.\n");
      return FALSE;
    }

  for (i = 0; i < args->argc; i++)
    {
      printf("Scanning file %s\n", args->argv[i]);
      
      if (! (trace = libnd_trace_new(args->argv[i])))
	{
	  printf("Could not open %s, skipping.\n", args->argv[i]);
	  continue;
	}

      ht = libnd_tcpconn_table_new();

      for (libnd_pit_init(&pit, trace); libnd_pit_get(&pit); libnd_pit_next(&pit))
	{
	  packet = libnd_pit_get(&pit);
	  libnd_tcpdump_get_packet_line(packet, line, FALSE);
	  new_conn = FALSE;

	  if (! (conn = libnd_tcpconn_table_lookup(ht, packet)))
	    {
	      new_conn = TRUE;
	      conn = libnd_tcpconn_new(packet);
	      libnd_tcpconn_table_add(ht, conn);
	    }

	  libnd_tcpconn_state_update(conn, packet);
	  printf("%s [%s%s]\n", line,
		 libnd_tcpconn_state_get_string(libnd_tcpconn_state_get(conn)),
		 new_conn ? "*" : "");
	}

      libnd_tcpconn_table_free(ht);
      libnd_trace_free(trace);
    }

  return TRUE;
  TOUCH(unused);
}

